#! /usr/bin/perl
#
# Movable Type Plug-in to generate QR code
# Copyright 2004 Daiji Hirata All Right Reserved.
# $Id: QRcode.pl,v 1.4 2004/08/12 09:10:06 hirata Exp $
#
# QRcode is trademark of Denso-Wave, see http://www.denso-wave.com/qrcode/ .

use strict;

my $PLUGIN_NAME = 'QRcode';
my $VERSION = 'v0.05';

my $cfg = {
    QRcode_filename_base => 'qrcode',
    QRcode_format => 'png',
    QRcode_path => '',
    QRcode_size => 3,
    QRcode_version => 4,
    QRcode_ecc => 'M',
};

use MT::Template::Context;

MT::Template::Context->add_tag(QRcode => \&generate_qrcode);

sub generate_qrcode {
    my $ctx = shift;
    my ($arg, $cond) = @_;

    my $blog = $ctx->stash('blog');
    my $entry = $ctx->stash('entry');

    require File::Spec;
    my ($url, $site_path, $title, $imagedir, $filename, $format, $size);
    if ($blog) {
	$url = $blog->site_url;
	$site_path = $blog->site_path;
	$title = $blog->name;
    } else {
	return $ctx->error(MT->translate('Cannot access stash of blog'));
    }

    if ($entry) {
	$url = $entry->permalink;
	$title = $entry->title || '';
    }

    $url = $arg->{url} || $url;
    $title = $arg->{title} || $title;
    $imagedir = $arg->{dir} || $cfg->{QRcode_path};
    $format = $arg->{format} || $cfg->{QRcode_format};
    $filename = $arg->{filename} . $entry ? '-' . $entry->id : '' || $cfg->{QRcode_filename_base};
    $filename .= '.' . lc $format;
    $size = $arg->{size} || $cfg->{QRcode_size};
    my $imagefile = File::Spec->catfile($site_path, $imagedir, $filename);
    my $imageurlpath = $imagedir ? File::Spec->catfile($imagedir, $filename) : $filename;

    unless (-e $imagefile && 
	    ( $arg->{cache} ne '0' &&
	      lc($arg->{cache}) ne 'off' &&
	      lc($arg->{cache}) ne 'no')) {
	require GD::Barcode::QRcode;
	my $msg = $url;
	if ($arg->{bookmark}) {
	    $msg = sprintf "MEBKM:TITLE:%s;URL:%s;;", _encode_qrcode($title), _encode_qrcode($url);
	}
	my $param = {
	    Version => $cfg->{QRcode_version},
	    Ecc => $cfg->{QRcode_ecc}, 
	    ModuleSize => $size,
	};
	my $qrcode = GD::Barcode::QRcode->new($msg, $param);
	my $image = $qrcode->plot()->$format;
	open my $fh, ">$imagefile" or return $ctx->error('cannot write file: ' . $imagefile);
	print $fh $image;
	close $fh;
    }
    return $imageurlpath;
}

sub _encode_qrcode {
    my $url = shift;
    $url =~ s!\\!\\\\!g;
    $url =~ s!:!\\:!g;
    $url =~ s!;!\\;!g;
    $url =~ s!,!\\,!g;
    return $url;
}

require MT::Plugin;
my $plugin = new MT::Plugin( { name => $PLUGIN_NAME . ' ' . $VERSION,
			       description => 'QRcode generator &lt;$MTQRcode$&gt;.',
			       dir_name => 'QRcode',
			       doc_link => 'http://uva.jp/dh/mt/qrcode.html',
			     } );

MT->add_plugin($plugin);
